import { CronJob } from "cron"
import logger from "@/logger.ts"

/**
 * Run a function every day at 1 AM, Oslo time.
 * @param onTick Function to run.
 * @returns CronJob instance.
 */
export function cronJobDaily(onTick: () => Promise<void>): CronJob {
  return CronJob.from({
    cronTime: "0 0 1 * * *",
    onTick: async () => {
      logger.info("Starting daily job")
      await onTick()
      logger.info("Finished daily job")
    },
    start: true,
    timeZone: "Europe/Oslo",
  })
}
